/*

    ThermostatQA - test framework for Thermostat Monitoring Tool

    Copyright 2013 Red Hat, Inc.

This file is part of ThermostatQA

ThermostatQA is distributed under the GNU General Public License,
version 2 or any later version (with a special exception described
below, commonly known as the "Classpath Exception").

A copy of GNU General Public License (GPL) is included in this
distribution, in the file COPYING.

Linking ThermostatQA code with other modules is making a combined work
based on ThermostatQA.  Thus, the terms and conditions of the GPL
cover the whole combination.

As a special exception, the copyright holders of ThermostatQA give you
permission to link this code with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on ThermostatQA code.  If you modify ThermostatQA, you may
extend this exception to your version of the software, but you are
not obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.thermostat.qa.framework;

import java.awt.AWTException;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.KeyEvent;
import java.io.IOException;
import java.lang.reflect.Method;

public abstract class ThermostatGuiTest extends ThermostatTest
{
    private static final String PATTERNS_DIRECTORY = "patterns";
    private static final String AA_SUBDIR = "AA";
    private static final String NOAA_SUBDIR = "noAA";

    private boolean useAntialiasedText = isTextAntialiasingOn();

    /**
     * Test if text antialiasing is used for rendering graphical user interface elements.
     *
     * @return true if system is sure that text antialiasing is used, false otherwise
     */
    protected static boolean isTextAntialiasingOn()
    {
        try
        {
            // Class sun.awt.SunToolkit is not part of Java standard API, so to compile
            // this tool it's needed to use reflections or to disable ct.sym file during
            // compilation.
            Class<?> sunToolkitClass = Class.forName("sun.awt.SunToolkit");
            Method getDesktopFontHintsMethod = sunToolkitClass.getDeclaredMethod("getDesktopFontHints", (Class<?>[]) null);
            RenderingHints renderingHints = ((RenderingHints) getDesktopFontHintsMethod.invoke(null));
            if (renderingHints == null)
            {
                return false;
            }
            Object textAntialiasing = renderingHints.get(RenderingHints.KEY_TEXT_ANTIALIASING);
            return textAntialiasing != RenderingHints.VALUE_TEXT_ANTIALIAS_OFF;
        }
        catch (Exception e)
        {
            return false;
        }
    }

    /**
     * Test if system font antialiasing setting is used for rendering graphical user interface elements.
     *
     * @return true if system font antialiasing setting is used, false otherwise
     */
    protected static boolean useSystemFontAASettings()
    {
        try
        {
            // Class sun.awt.SunToolkit is not part of Java standard API, so to compile
            // this tool it's needed to use reflections or to disable ct.sym file during
            // compilation.
            Class<?> sunToolkitClass = Class.forName("sun.awt.SunToolkit");
            Method useSystemAAFontSettingsMethod = sunToolkitClass.getDeclaredMethod("useSystemAAFontSettings", (Class<?>[]) null);
            // make this private method callable
            useSystemAAFontSettingsMethod.setAccessible(true);
            return ((Boolean) useSystemAAFontSettingsMethod.invoke(null)).booleanValue();
        }
        catch (Exception e)
        {
            return false;
        }
    }

    /**
     * Start Thermostat GUI client in a new thread.
     */
    protected void runThermostatGuiInANewThread()
    {
        new GuiThread().start();
        logInfo("runThermostatGuiInANewThread", "Thermostat GUI started");
    }

    /**
     * Start Thermostat storage.
     */
    protected GuiRobot startStorageOnly() throws AWTException
    {
        GuiRobot robot = new GuiRobot();
        startStorage();
        return robot;
    }

    /**
     * Start Thermostat storage and also Thermostat GUI client in a new thread.
     */
    protected GuiRobot startThermostatAndStorage(String screenshotName) throws AWTException, IOException
    {
        GuiRobot robot = new GuiRobot();

        if (this.configuration.useDummyGfxTests())
        {
            robot.loadScreenshot(screenshotName);
        }
        else
        {
            startStorage();
            runThermostatGuiInANewThread();
            sleep(FIVE_SEC);
        }
        return robot;
    }

    /**
     * Start Thermostat web storage and also Thermostat GUI client in a new
     * thread.
     */
    protected GuiRobot startThermostatAndWebStorage(String screenshotName) throws AWTException, IOException
    {
        GuiRobot robot = new GuiRobot();

        if (this.configuration.useDummyGfxTests())
        {
            robot.loadScreenshot(screenshotName);
        }
        else
        {
            startWebStorage();
            runThermostatGuiInANewThread();
            sleep(FIVE_SEC);
        }
        return robot;
    }
    
    /**
     * Try to find some pattern on the screenshot.
     *
     * @return rectangle with pattern coordinates if pattern is found, null otherwise.
     */
    protected Rectangle findPattern(GuiRobot robot, String patternFileName, String patternName) throws IOException
    {
        logInfo("checkForPattern", "looking for pattern " + patternName);

        // patterns should be divided into two categories: one for desktop with text AA enabled
        // and one for desktop with AA disabled
        String subdir = this.configuration.getThermostatVersion()+"/"+
                        (this.useAntialiasedText ? AA_SUBDIR : NOAA_SUBDIR);
        String fullPatternName = PATTERNS_DIRECTORY + "/" + subdir + "/" + patternFileName + ".png";

        return robot.findPattern(fullPatternName);
    }

    /**
     * Try to find some pattern on the screenshot.
     *
     * @return rectangle with pattern coordinates if pattern is found, null otherwise.
     */
    protected Rectangle findPattern(GuiRobot robot, String[] patternFileNames, String patternName) throws IOException
    {
        Rectangle result = null;
        int version = 1;
        for (String pattern : patternFileNames)
        {
            result = findPattern(robot, pattern, patternName + " version " + version);
            version++;
            if (result != null)
                break;
        }
        return result;
    }

    /**
     * Check if some pattern can really be located on a screenshot.
     *
     * @return rectangle with pattern coordinates if pattern is found, null otherwise.
     */
    protected Rectangle checkForPattern(GuiRobot robot, String patternFileName, String patternName) throws IOException
    {
        Rectangle rectangle = findPattern(robot, patternFileName, patternName);

        if (rectangle == null)
        {
            stopThermostatAndStorage(robot);
        }
        else
        {
            logInfo("checkForPattern", "pattern found at " + rectangle.toString());
        }
        Assert.assertNotNull(rectangle, "Error: " + patternName + " pattern can't be found!");
        robot.highlightRectangle(rectangle);
        return rectangle;
    }

    /**
     * Check if one of the patterns can really be located on a screenshot.
     *
     * @return rectangle with pattern coordinates if pattern is found, null otherwise.
     */
    protected Rectangle checkForPattern(GuiRobot robot, String[] patternFileNames, String patternName) throws IOException
    {
        Rectangle rectangle = findPattern(robot, patternFileNames, patternName);

        if (rectangle == null)
        {
            stopThermostatAndStorage(robot);
        }
        else
        {
            logInfo("checkForPattern", "pattern found at " + rectangle.toString());
        }
        Assert.assertNotNull(rectangle, "Error: " + patternName + " pattern can't be found!");
        robot.highlightRectangle(rectangle);
        return rectangle;
    }

    /**
     * Stop Thermostat storage and also Thermostat GUI client.
     */
    protected void stopThermostatAndStorage(GuiRobot robot) throws IOException
    {
        if (this.configuration.useRealGfxTests())
        {
            stopStorage();
            stopGUI(robot);
        }
    }

    /**
     * Stop Thermostat web storage and also Thermostat GUI client.
     */
    protected void stopThermostatAndWebStorage(GuiRobot robot) throws IOException
    {
        if (this.configuration.useRealGfxTests())
        {
            stopWebStorage();
            stopGUI(robot);
        }
    }
    
    /**
     * Create screenshot after some slight delay.
     */
    protected void createScreenshot(GuiRobot robot, String screenshotName) throws IOException
    {
        sleep(ONE_SEC);
        robot.createScreenCapture();
        robot.saveScreenshot(screenshotName);
    }

    /**
     * Move focus to the Thermostat GUI client main menu by simulating F10 key press action.
     */
    protected void enterMainMenu(GuiRobot robot)
    {
        robot.pressKey(KeyEvent.VK_F10);
    }

}
